#ifndef  _GRAFAPSOLUTE_H_IS_INCLUDED_
#define  _GRAFAPSOLUTE_H_IS_INCLUDED_
#include <iostream>
#include <list>
#include <vector>
#include <utility>
#include <fstream>
#include <string>
#include <ctime>
#include <sstream> 
#include <algorithm>
#include <numeric>
#include <typeinfo>
#include <cstdlib>
#include <math.h> //dodamo zbof devc++


using namespace std;

#define NAME_PRINT(x) cout << #x << endl

class GraphApsolute
{
	string inputFileName;
	int numberV; //broj vrhova
	int numberE; //broj bridova
	int numberS; //broj scenarija
	int standard_problem; //ako je -1 onda rijesava robust, ako je neki broj pozitivni, onda rijesi taj scenario

	vector<vector<int> >(GraphApsolute::*populationPtr)(void);
	vector<int>(GraphApsolute::*crossoverPtr)(vector<int>&, vector<int>&);
	vector<int>(GraphApsolute::*recoverPtr)(vector<int>&);
	vector<int>(GraphApsolute::*mutationPtr)(vector<int>);

	vector<vector<bool> > adjacencyMatrix;
	vector<vector<int> > allScenarios;

	vector<int> numberOfNeighbours;
	vector<int> contribution;
	vector<int> allVerticesSortedByContribution;
	vector<int> profits;
	vector<int> allVerticesSortedByProfit;

	void addEdge(int u, int v);
	void addScenario(vector <int>& oneScenario);
	
public:

	GraphApsolute(string fileName, int standard);

	void printGraph();
	void printScenarios();

	bool isConected(int vertex, vector<int>& someSet);
	int isElement(int vertex, vector<int>& someSet); //vraca poziciju, ako nema -1

	int calculateProfitAtScenario(vector<int>& someVector, int scenario);
	int calculateMinimalProfit(vector<int>& someVector);

	pair<vector<int>, int> greedyAlgorithmStandardProblem(vector<int> vertices, vector<int> criterium); 

	pair<vector<int>, int> greedyAlgorithmProfit();
	pair<vector<int>, int> greedyAlgorithmContribution();
	pair<vector<int>, int> geneticAlgorithm(int populationType, int crossovertype, int mutationType, int recoverType, int numberOfSteps, int goodTournamentSize, int badTournamentSize, int mutationParameter); //mutacijski parametar ide u postotcima
	
	vector<vector<int> > createPopulationGreedyContribution();
	vector<vector<int> > createPopulationGreedyProfit();
	vector<vector<int> > createPopulationExactSolutions();
	vector<vector<int> > createPopulationMix();
	void fixPopulation(vector<vector<int> >& initialPopulation);

	//vracamo uvijek poziciju jedinke tako da imamo mogucnosti kasnije bas nju maknuti ili nesto
	pair<int, int> goodTournament(vector<vector<int> >& initialPopulation, int tournamentSize);
	int badTournament (vector<vector<int> >& initialPopulation, int tournamentSize); //vrati indeks najgoreg
	int similarity(vector<int> newChild, vector<vector <int> >& initialPopulation, double deviation); //vrati indeks slicnog

	vector<int> feasibilityRecoverGreedyContribution(vector<int>& someSolution);
	vector<int> feasibilityRecoverGreedyContributionAndRandom(vector<int>& someSolution);
	vector<int> feasibilityRecoverGreedyProfit(vector<int>& someSolution);
	vector<int> feasibilityRecoverGreedyProfitAndRandom(vector<int>& someSolution);

	vector<int> crossoverGreedyContributionAltrenateCombine(vector<int>& parent1, vector<int>& parent2);
	vector<int> crossoverGreedyContributionAltrenateCombineLocal(vector<int>& parent1, vector<int>& parent2);
	vector<int> crossoverGreedyContributionAllVertices(vector<int>& parent1, vector<int>& parent2);
	vector<int> crossoverGreedyApsoluteProfitAllVertices(vector<int>& parent1, vector<int>& parent2);

	vector<int> mutationDifferenceContribution(vector<int> someSolution);
	vector<int> mutationLocalSearch(vector<int> someSolution);
	vector<int> mutationLowProfit(vector<int> someSolution);
	vector<int> mutationHighProfit(vector<int> someSolution);
	vector<int> mutationSwap(vector<int> someSolution);
	vector<int> mutationMix(vector<int> someSolution);
};

void quickSort(vector<int>& vertices, vector<int>& profits, const int left, const int right, const int size);
int partition(vector<int>& vertices, vector<int>& profits, const int left, const int right);

void printVectorInt(vector<int>& someVector); //jer necemo moci promijeniti referencu

double runAbsoluteExample(string baseFileName, int populationType, int crossovertype, int mutationType, int recoverType, int numberOfSteps, int goodTournamentSize, int badTournamentSize, int mutationParameter); //mutacijski parametar ide u postotcima
void runStandardExample(string baseFileName, int populationType, int crossovertype, int mutationType, int recoverType, int numberOfSteps, int goodTournamentSize, int badTournamentSize, int mutationParameter); //mutacijski parametar ide u postotcima
void check();

#endif
