#include "GraphrelativeDeviation.h"

GraphRelativeDeviation::GraphRelativeDeviation(string fileName)
{
	ifstream file;
	int i, j, left, right;
	char pom;

	file.open(fileName);

	if (!file.is_open())
		throw "Ne mogu otvoriti datoteku";

	inputFileName = fileName;

	vector<vector<int> >(GraphRelativeDeviation::*populationPtr)(void) = NULL;
	vector<int>(GraphRelativeDeviation::*crossoverPtr)(vector<int>&, vector<int>&) = NULL;
	vector<int>(GraphRelativeDeviation::*mutationPtr)(vector<int>) = NULL;
	vector<int>(GraphRelativeDeviation::*recoverPtr)(vector<int>&) = NULL;

	//prvi red datoteke izgleda: brojVrhova brojBridova brojScenarija
	file >> numberV >> numberE >> numberS;

	vector<vector<bool> > neighbours(numberV, vector<bool>(numberV, false));
	adjacencyMatrix = neighbours;
	//za scenarije radi defaultni kontruktor

	//pomvec je pomocni vektor za postavljanj vektora na nul
	vector<int> pomvec(numberV, 0);

	numberOfNeighbours = pomvec;
	allVerticesSortedByContribution = pomvec;
	contribution = pomvec;
	allVerticesSortedByProfit = pomvec;
	profits = pomvec;

	iota(allVerticesSortedByContribution.begin(), allVerticesSortedByContribution.end(), 0);
	iota(allVerticesSortedByProfit.begin(), allVerticesSortedByProfit.end(), 0);
	// iota daje vrijednosti u nekom rasponu, dakle imamo niz: 0, 1, 2, ..., numberV-1;

	//krecu retci s bridovima: broj broj
	for (i = 0; i < numberE; ++i)
	{
		file >> pom >> left >> right; //temp treba kad imamo njihove
		this->addEdge(left, right); // -1 treba kad stavljamo njihove failove jer ide od 1
	}

	//krecu retci sa scenarijima, ocitavamo jedan po jedan redak
	for (i = 0; i < numberS; ++i)
	{
		for (j = 0; j < numberV; ++j)
		{
			file >> pomvec[j];
			//ujedno racunamo i profit tj doprinos
			profits[j] += pomvec[j];
		}

		this->addScenario(pomvec);
	}

	contribution = profits;

	for (i = 0; i < numberV; ++i)
	{
		if (numberOfNeighbours[i] != 0)
			contribution[i] = int((double)contribution[i] / numberOfNeighbours[i] + 0.5);
		//contribution[i] = int((double)contribution[i] / (numberOfNeighbours[i] + 1));
	}

	quickSort(allVerticesSortedByContribution, contribution, 0, allVerticesSortedByContribution.size() - 1, allVerticesSortedByContribution.size());
	quickSort(allVerticesSortedByProfit, profits, 0, allVerticesSortedByProfit.size() - 1, allVerticesSortedByProfit.size());

	ifstream solFile, realFile;
	string str, profitStr, solFileName = "G_standard_solutions_" + fileName, realFileName = "standard_solutions_" + fileName;
	double profit;

	solFile.open(solFileName);
	realFile.open(realFileName);

	if (!solFile.is_open())
		throw "Ne mogu otvoriti datoteku";

	if (!realFile.is_open())
		throw "Ne mogu otvoriti datoteku";

	for (i = 0; i < numberS; ++i)
	{
		getline(solFile, str);
		str = str.substr(str.find(",") + 1, str.length()); //x-ovi, iznos, vrijeme
		profitStr = str.substr(str.find(",") + 1, str.length());
		profit = stod(profitStr.substr(0, profitStr.find(",")));
		str = str.substr(0, str.find(","));

		string delimiter = " ";
		size_t pos = 0; //moze biti i nedeklarirano
		string token;
		vector<int> oneSolution;
		pair<vector<int>, double> onePair;


		while ((pos = str.find(delimiter)) != string::npos)
		{
			token = str.substr(0, pos); //drugi parametar je duljina koju hocemo
			str.erase(0, pos + delimiter.length());
			oneSolution.push_back(stoi(token));
		}

		onePair.first = oneSolution;
		onePair.second = profit;
		bestSolutionsAtScenariosAndProfits.push_back(onePair);

		getline(realFile, str);
		str = str.substr(str.find(",") + 1, str.length()); //x-ovi, iznos, vrijeme
		profitStr = str.substr(str.find(",") + 1, str.length());
		profit = stod(profitStr.substr(0, profitStr.find(",")));

		profitsReal.push_back(profit);
	}

	file.close();
	solFile.close();
	realFile.close();
}

void GraphRelativeDeviation::addEdge(int u, int v)
{
	adjacencyMatrix[u][v] = true;
	adjacencyMatrix[v][u] = true;

	numberOfNeighbours[u]++;
	numberOfNeighbours[v]++;
}

void GraphRelativeDeviation::addScenario(vector <int>& oneScenario)
{
	allScenarios.push_back(oneScenario);
}

void GraphRelativeDeviation::printGraph()
{
	int i, j;

	for (i = 0; i < numberV; ++i)
	{
		for (j = 0; j < numberV; ++j)
			cout << adjacencyMatrix[i][j] << " ";
		cout << endl;
	}
	cout << endl;
}

void GraphRelativeDeviation::printScenarios()
{
	int i, j;

	for (i = 0; i < numberS; ++i)
	{
		for (j = 0; j < numberV; ++j)
			cout << allScenarios[i][j] << " ";
		cout << endl;
	}
	cout << endl;
}

bool GraphRelativeDeviation::isConected(int vertex, vector <int>&someSet)
{
	int i;

	for (i = 0; i < someSet.size(); ++i)
		if (adjacencyMatrix[vertex][someSet.at(i)])
			return true;

	return false;
}

int GraphRelativeDeviation::isElement(int vertex, vector <int>&someSet)
{
	int i;

	for (i = 0; i < someSet.size(); ++i)
	{
		if (someSet.at(i) == vertex)
			return i;
	}

	return -1;
}

double GraphRelativeDeviation::calculateRelativeDeviationAtScenario(vector<int>& someVector, int scenario)
{
	int i;
	double profit = 0;

	for (i = 0; i < someVector.size(); ++i)
		profit += allScenarios[scenario][someVector.at(i)];

	//cout << bestSolutionsAtScenariosAndProfits[scenario].second - profit << endl;

	return  (bestSolutionsAtScenariosAndProfits[scenario].second - profit)/ bestSolutionsAtScenariosAndProfits[scenario].second;
}

double GraphRelativeDeviation::calculateRelativeDeviationAtScenarioReal(vector<int>& someVector, int scenario)
{
	int i;
	double profit = 0;

	for (i = 0; i < someVector.size(); ++i)
		profit += allScenarios[scenario][someVector.at(i)];

	//cout << bestSolutionsAtScenariosAndProfits[scenario].second - profit << endl;

	return  (profitsReal[scenario] - profit) / profitsReal[scenario];
}

double GraphRelativeDeviation::calculateMaximalRelativeDeviation(vector<int>& someVector)
{
	int i;
	double temp, worst = calculateRelativeDeviationAtScenario(someVector, 0);

	for (i = 1; i < numberS; ++i)
	{
		temp = calculateRelativeDeviationAtScenario(someVector, i);
		if (temp > worst)
			worst = temp;
	}

	return worst;
}

double GraphRelativeDeviation::calculateMaximalRelativeDeviationReal(vector<int>& someVector)
{
	int i;
	double temp, worst = calculateRelativeDeviationAtScenarioReal(someVector, 0);

	for (i = 1; i < numberS; ++i)
	{
		temp = calculateRelativeDeviationAtScenarioReal(someVector, i);
		if (temp > worst)
			worst = temp;
	}

	return worst;
}

vector<vector<int> > GraphRelativeDeviation::createPopulationExactSolutions()
{
	int i;
	vector < vector<int> > initialPopulation;

	//imamo spremljeno u parovima bestSolutionsAtScenariosAndProfits
	for (i = 0; i < numberS; ++i)
		initialPopulation.push_back(bestSolutionsAtScenariosAndProfits[i].first);

	return initialPopulation;
}

vector<vector<int> > GraphRelativeDeviation::createPopulationGreedyContribution()
{
	int i, j;
	vector < vector<int> > initialPopulation;
	vector<int> allVertices(numberV, 0);
	iota(allVertices.begin(), allVertices.end(), 0);

	for (i = 0; i < numberS; ++i)
	{
		vector<int> pom = allScenarios[i];
		//prebaciti na double?!?!? lose sve 0,1 ili 2?!!?!?!?!?!?

		for (j = 0; j < numberV; ++j)
			if (numberOfNeighbours[j] != 0)
				pom[j] = (int)((double)pom[j] / numberOfNeighbours[j] + 0.5);

		initialPopulation.push_back(greedyAlgorithmStandardProblem(allVertices, pom).first);
	}

	return initialPopulation;
}

vector<vector<int> > GraphRelativeDeviation::createPopulationGreedyProfit()
{
	int i;
	vector < vector<int> > initialPopulation;
	vector<int> allVertices(numberV, 0);
	iota(allVertices.begin(), allVertices.end(), 0);

	for (i = 0; i < numberS; ++i)
		initialPopulation.push_back(greedyAlgorithmStandardProblem(allVertices, allScenarios[i]).first);

	return initialPopulation;
}

vector<vector<int> > GraphRelativeDeviation::createPopulationMix()
{
	vector < vector<int> > initialPopulation = createPopulationExactSolutions();

	vector < vector<int> > pom = createPopulationGreedyContribution();
	initialPopulation.insert(initialPopulation.begin(), pom.begin(), pom.end());

	pom = createPopulationGreedyProfit();
	initialPopulation.insert(initialPopulation.begin(), pom.begin(), pom.end());

	return initialPopulation;
}

int GraphRelativeDeviation::similarity(vector<int> newChild, vector<vector <int> >& initialPopulation, double deviation)
{
	int i;
	double tempProfit, childProfit;

	childProfit = calculateMaximalRelativeDeviation(newChild);

	for (i = 0; i < initialPopulation.size(); ++i)
	{
		tempProfit = calculateMaximalRelativeDeviation(initialPopulation.at(i));

		if ((tempProfit / childProfit > 1) && (tempProfit / childProfit < (1 + deviation)))
			return i;
	}

	return -1; //nema slicnog, vrati -1 
}

int GraphRelativeDeviation::badTournament(vector<vector <int> >& initialPopulation, int tournamentSize)
{
	int i, temp, worst;
	double worstRelativeDeviation, tempRelativeDeviation;
	vector<int> pool;

	for (i = 0; i < tournamentSize; ++i)
	{
		temp = (int)rand() % (initialPopulation.size()); //vraca nam broj redaka ako radimo size na matricu

		while (isElement(temp, pool) != -1) //dok ne dobijemo novog
			temp = (int)rand() % (initialPopulation.size());

		pool.push_back(temp);
	}

	worst = pool.at(0);
	worstRelativeDeviation = calculateMaximalRelativeDeviation(initialPopulation.at(worst));

	//ne mozemo odmah usporedivati i zamijenjivati da ne ponovimo nekog npr 1. i 3. put dode isti
	for (i = 1; i < tournamentSize; ++i)
	{
		tempRelativeDeviation = calculateMaximalRelativeDeviation(initialPopulation.at(pool.at(i)));

		if (tempRelativeDeviation > worstRelativeDeviation)
		{
			worstRelativeDeviation = tempRelativeDeviation;
			worst = pool.at(i);
		}
	}

	return worst;
}

pair<int, int> GraphRelativeDeviation::goodTournament(vector<vector <int> >& initialPopulation, int tournamentSize)
{
	int i, first, second, temp;
	double firstRelativeDeviation, secondRelativeDeviation, tempRelativeDeviation;
	vector<int> pool;

	for (i = 0; i < tournamentSize; ++i)
	{
		temp = (int)rand() % (initialPopulation.size()); //vraca nam broj redaka ako radimo size na matricu

		while (isElement(temp, pool) != -1) //dok ne dobijemo novog
			temp = (int)rand() % (initialPopulation.size());

		pool.push_back(temp);
	}

	first = pool.at(0);
	second = pool.at(1);

	firstRelativeDeviation = calculateMaximalRelativeDeviation(initialPopulation.at(first));
	secondRelativeDeviation = calculateMaximalRelativeDeviation(initialPopulation.at(second));

	if (firstRelativeDeviation > secondRelativeDeviation) //stavit cemo da je 1. roditelj jaci zbog if-ova koji slijede
	{
		swap(first, second);
		swap(firstRelativeDeviation, secondRelativeDeviation);
	}

	//opet, ne mozemo odmah usporedivati i zamijenjivati da ne ponovimo nekog
	for (i = 2; i < tournamentSize; ++i)
	{
		//sad nam je temp redni broj promatranog rjesenja
		temp = pool.at(i);
		tempRelativeDeviation = calculateMaximalRelativeDeviation(initialPopulation.at(temp));

		if (tempRelativeDeviation < firstRelativeDeviation)
		{
			second = first;
			secondRelativeDeviation = firstRelativeDeviation;
			first = temp;
			firstRelativeDeviation = tempRelativeDeviation;
		}
		else if (tempRelativeDeviation < secondRelativeDeviation)
		{
			second = temp;
			secondRelativeDeviation = tempRelativeDeviation;
		}
	}
	pair <int, int> parents;

	if ((int)rand() % 2 == 0) //da ne vracamo jaceg uvijek prvog
	{
		parents.first = first;
		parents.second = second;
		return parents;
	}
	else
	{
		parents.first = second;
		parents.second = first;
		return parents;
	}
}

vector<int> GraphRelativeDeviation::feasibilityRecoverGreedyContribution(vector<int>& someSolution)
{
	//prvo provjerimo da li je dopustivo pa nadopunimo

	int i, j;
	vector<int> recoveredSolution, someSolutionContribution(someSolution.size(), 0);

	for (i = 0; i < numberS; ++i)
		for (j = 0; j < someSolution.size(); ++j)
			someSolutionContribution[j] += allScenarios[i][someSolution.at(j)];

	//ako nema susjeda ostavi kako je
	for (i = 0; i < someSolution.size(); ++i)
		if (numberOfNeighbours[someSolution.at(i)] != 0)
			someSolutionContribution[i] = int((double)someSolutionContribution[i] / numberOfNeighbours[someSolution.at(i)] + 0.5);

	quickSort(someSolution, someSolutionContribution, 0, someSolution.size() - 1, someSolution.size());

	//provjera jel dopustivo
	//niz je padajuce sortiran po doprinosu
	for (i = 0; i < someSolution.size(); ++i)
		if (!isConected(someSolution.at(i), recoveredSolution) && (isElement(someSolution.at(i), recoveredSolution) == -1))
			recoveredSolution.push_back(someSolution.at(i));

	//mozda mozemo jos neki dodati ako nije vec u recoverd solution i nije povezan s ostalima
	//allVerticesSoretedByContribution je sortiran padajuce  po doprinosu
	for (i = 0; i < numberV; ++i)
		if ((!isConected(allVerticesSortedByContribution.at(i), recoveredSolution)) && (isElement(allVerticesSortedByContribution.at(i), recoveredSolution) == -1))
			recoveredSolution.push_back(allVerticesSortedByContribution.at(i));

	return recoveredSolution;
}

vector<int> GraphRelativeDeviation::feasibilityRecoverGreedyProfit(vector<int>& someSolution)
{
	//prvo provjerimo da li je dopustivo pa nadopunimo

	int i, j;
	vector<int> recoveredSolution, someSolutionProfit(someSolution.size(), 0);

	for (i = 0; i < numberS; ++i)
		for (j = 0; j < someSolution.size(); ++j)
			someSolutionProfit[j] += allScenarios[i][someSolution.at(j)];

	quickSort(someSolution, someSolutionProfit, 0, someSolution.size() - 1, someSolution.size());

	//provjera jel dopustivo
	//niz je padajuce sortiran po profitu
	for (i = 0; i < someSolution.size(); ++i)
		if (!isConected(someSolution.at(i), recoveredSolution) && (isElement(someSolution.at(i), recoveredSolution) == -1))
			recoveredSolution.push_back(someSolution.at(i));

	//mozda mozemo jos neki dodati ako nije vec u recoverd solution i nije povezan s ostalima
	//allVerticesSoretedByProfit je sortiran padajuce po profitu
	for (i = 0; i < numberV; ++i)
		if ((!isConected(allVerticesSortedByProfit.at(i), recoveredSolution)) && (isElement(allVerticesSortedByProfit.at(i), recoveredSolution) == -1))
			recoveredSolution.push_back(allVerticesSortedByProfit.at(i));

	return recoveredSolution;
}

vector<int> GraphRelativeDeviation::feasibilityRecoverGreedyContributionAndRandom(vector<int>& someSolution)
{
	//prvo provjerimo da li je dopustivo pa nadopunimo
	//pocetna provjera je po contributionu, nadopuna random

	int i, j;
	vector<int> recoveredSolution, someSolutionContribution(someSolution.size(), 0);

	for (i = 0; i < numberS; ++i)
		for (j = 0; j < someSolution.size(); ++j)
			someSolutionContribution[j] += allScenarios[i][someSolution.at(j)];

	//ako nema susjeda ostavi kako je
	for (i = 0; i < someSolution.size(); ++i)
		if (numberOfNeighbours[someSolution.at(i)] != 0)
			someSolutionContribution[i] = int((double)someSolutionContribution[i] / numberOfNeighbours[someSolution.at(i)] + 0.5);

	quickSort(someSolution, someSolutionContribution, 0, someSolution.size() - 1, someSolution.size());

	//provjera jel dopustivo
	//niz je padajuce sortiran po doprinosu
	for (i = 0; i < someSolution.size(); ++i)
		if (!isConected(someSolution.at(i), recoveredSolution) && (isElement(someSolution.at(i), recoveredSolution) == -1))
			recoveredSolution.push_back(someSolution.at(i));

	vector<int> allVerticesShuffle(numberV, 0);
	iota(allVerticesShuffle.begin(), allVerticesShuffle.end(), 0);
	random_shuffle(allVerticesShuffle.begin(), allVerticesShuffle.end());

	//mozda mozemo jos neki dodati ako nije vec u recoverd solution i nije povezan s ostalima
	for (i = 0; i < numberV; ++i)
		if ((!isConected(allVerticesShuffle.at(i), recoveredSolution)) && (isElement(allVerticesShuffle.at(i), recoveredSolution) == -1))
			recoveredSolution.push_back(allVerticesShuffle.at(i));

	return recoveredSolution;
}

vector<int> GraphRelativeDeviation::feasibilityRecoverGreedyProfitAndRandom(vector<int>& someSolution)
{
	//prvo provjerimo da li je dopustivo pa nadopunimo
	//pocetna provjera je po profitu, nadopuna random

	int i, j;
	vector<int> recoveredSolution, someSolutionProfit(someSolution.size(), 0);

	for (i = 0; i < numberS; ++i)
		for (j = 0; j < someSolution.size(); ++j)
			someSolutionProfit[j] += allScenarios[i][someSolution.at(j)];

	quickSort(someSolution, someSolutionProfit, 0, someSolution.size() - 1, someSolution.size());

	//provjera jel dopustivo
	//niz je padajuce sortiran po doprinosu
	for (i = 0; i < someSolution.size(); ++i)
		if (!isConected(someSolution.at(i), recoveredSolution) && (isElement(someSolution.at(i), recoveredSolution) == -1))
			recoveredSolution.push_back(someSolution.at(i));

	vector<int> allVerticesShuffle(numberV, 0);
	iota(allVerticesShuffle.begin(), allVerticesShuffle.end(), 0);
	random_shuffle(allVerticesShuffle.begin(), allVerticesShuffle.end());

	//mozda mozemo jos neki dodati ako nije vec u recoverd solution i nije povezan s ostalima
	for (i = 0; i < numberV; ++i)
		if ((!isConected(allVerticesShuffle.at(i), recoveredSolution)) && (isElement(allVerticesShuffle.at(i), recoveredSolution) == -1))
			recoveredSolution.push_back(allVerticesShuffle.at(i));

	return recoveredSolution;
}

vector<int> GraphRelativeDeviation::crossoverGreedyContributionAllVertices(vector<int>& parent1, vector<int>& parent2)
{
	vector<int> child;
	int i, j;
	vector<int> longChild(numberV, 0);
	int sum1 = 0, sum2 = 0;
	double probability1, probability2;
	double randomValue;

	//ako oboje imaju isti gen tada je rez=1
	//ako samo roditelj1 ima gen tada je rez=-1
	//ako samo roditelj2 ima gen tada je rez=2
	for (i = 0; i < parent1.size(); ++i)
		longChild.at(parent1.at(i)) = -1;

	for (i = 0; i < parent2.size(); ++i)
		longChild.at(parent2.at(i)) = 2;

	for (i = 0; i < numberS; ++i)
		for (j = 0; j < parent1.size(); ++j)
			sum1 += allScenarios[i][parent1.at(j)];

	for (i = 0; i < numberS; ++i)
		for (j = 0; j < parent2.size(); ++j)
			sum2 += allScenarios[i][parent2.at(j)];

	for (i = 0; i < parent1.size(); ++i)
		if (numberOfNeighbours[parent1.at(i)] != 0)
			sum1 = sum1 / numberOfNeighbours[parent1.at(i)];

	for (i = 0; i < parent2.size(); ++i)
		if (numberOfNeighbours[parent2.at(i)] != 0)
			sum2 = sum2 / numberOfNeighbours[parent2.at(i)];

	probability1 = (double)sum1 / (sum1 + sum2);
	probability2 = (double)sum2 / (sum1 + sum2);

	for (i = 0; i < numberV; ++i)
	{
		if (longChild.at(i) == 1)
			child.push_back(i);
		else
		{
			randomValue = (double)rand() / RAND_MAX;

			if (probability1 <= probability2)
			{
				if (randomValue < probability1) //nasljeduje se gen od roditelja1
				{
					if (longChild.at(i) == -1)
						child.push_back(i);
				}
				else //nasljeduje se gen od roditelja2
				{
					if (longChild.at(i) == 2)
						child.push_back(i);
				}
			}
			else
			{
				if (randomValue < probability2) //nasljeduje se gen od roditelja1
				{
					if (longChild.at(i) == 2)
						child.push_back(i);
				}
				else //nasljeduje se gen od roditelja2
				{
					if (longChild.at(i) == -1)
						child.push_back(i);
				}
			}
		}
	}

	child = (*this.*recoverPtr)(child);

	return child;
}

vector<int> GraphRelativeDeviation::crossoverGreedyContributionAltrenateCombine(vector<int>& parent1, vector<int>& parent2)
{
	//naizmjenicno dodaje elemente iz prvog i drugog roditelja

	int i, j, ipos, jpos;
	bool added, jadded, iadded;
	vector<int> child;

	//kreirali first i second da ne mijenjamo roditelje poslane u funkciju
	vector<int> first = parent1;
	vector<int> second = parent2;
	vector<int> firstContribution(parent1.size(), 0);
	vector<int> secondContribution(parent2.size(), 0);

	for (i = 0; i < numberS; ++i)
	{
		for (j = 0; j < first.size(); ++j)
			firstContribution[j] += allScenarios[i][first.at(j)];

		for (j = 0; j < second.size(); ++j)
			secondContribution[j] += allScenarios[i][second.at(j)];
	}

	for (i = 0; i < first.size(); ++i)
		if (numberOfNeighbours[first.at(i)] != 0)
			firstContribution[i] = int((double)firstContribution[i] / numberOfNeighbours[first.at(i)] + 0.5);

	for (i = 0; i < second.size(); ++i)
		if (numberOfNeighbours[second.at(i)] != 0)
			secondContribution[i] = int((double)secondContribution[i] / numberOfNeighbours[second.at(i)] + 0.5);

	quickSort(first, firstContribution, 0, first.size() - 1, first.size());
	quickSort(second, secondContribution, 0, second.size() - 1, second.size());

	//da ne moramo svaki put ispocetka kretati gledati niz
	ipos = jpos = 0;

	//krece dodavanje
	do
	{
		added = iadded = jadded = false;

		for (i = ipos; i < first.size(); ++i)
			if (!isConected(first.at(i), child) && (isElement(first.at(i), child) == -1))
			{
				child.push_back(first.at(i));
				ipos = i + 1;
				added = iadded = true;
				break;
			}

		//inace nije nasao nikog novog za dodati
		if (!iadded)
			ipos = first.size();

		for (j = jpos; j < second.size(); ++j)
			if (!isConected(second.at(j), child) && (isElement(second.at(j), child) == -1))
			{
				child.push_back(second.at(j));
				jpos = j + 1;
				added = jadded = true;
				break;
			}

		//inace nije nasao nikog novog za dodati
		if (!jadded)
			jpos = second.size();

	} while (added);

	child = (*this.*recoverPtr)(child);

	return child;
}

vector<int> GraphRelativeDeviation::crossoverGreedyContributionAltrenateCombineLocal(vector<int>& parent1, vector<int>& parent2)
{
	//naizmjenicno dodaje elemente iz prvog i drugog roditelja

	int i, j, ipos, jpos;
	bool added, jadded, iadded;
	vector<int> child;

	//kreirali first i second da ne mijenjamo roditelje poslane u funkciju
	vector<int> first = parent1;
	vector<int> second = parent2;
	vector<int> firstContribution(parent1.size(), 0), firstNumberOfNeighbours(parent1.size(), 0);
	vector<int> secondContribution(parent2.size(), 0), secondNumberOfNeighbours(parent2.size(), 0);

	for (i = 0; i < parent1.size(); ++i)
		for (j = 0; j < parent2.size(); ++j)
			if (adjacencyMatrix[parent1.at(i)][parent2.at(j)])
			{
				++firstNumberOfNeighbours[i];
				++secondNumberOfNeighbours[j];
			}

	for (i = 0; i < numberS; ++i)
	{
		for (j = 0; j < first.size(); ++j)
			firstContribution[j] += allScenarios[i][first.at(j)];

		for (j = 0; j < second.size(); ++j)
			secondContribution[j] += allScenarios[i][second.at(j)];
	}

	for (i = 0; i < first.size(); ++i)
		if (firstNumberOfNeighbours[i] != 0)
			firstContribution[i] = int((double)firstContribution[i] / firstNumberOfNeighbours[i] + 0.5);

	for (i = 0; i < second.size(); ++i)
		if (secondNumberOfNeighbours[i] != 0)
			secondContribution[i] = int((double)secondContribution[i] / secondNumberOfNeighbours[i] + 0.5);

	quickSort(first, firstContribution, 0, first.size() - 1, first.size());
	quickSort(second, secondContribution, 0, second.size() - 1, second.size());

	//da ne moramo svaki put ispocetka kretati gledati niz
	ipos = jpos = 0;

	//krece dodavanje
	do
	{
		added = iadded = jadded = false;

		for (i = ipos; i < first.size(); ++i)
			if (!isConected(first.at(i), child) && (isElement(first.at(i), child) == -1))
			{
				child.push_back(first.at(i));
				ipos = i + 1;
				added = iadded = true;
				break;
			}

		//inace nije nasao nikog novog za dodati
		if (!iadded)
			ipos = first.size();

		for (j = jpos; j < second.size(); ++j)
			if (!isConected(second.at(j), child) && (isElement(second.at(j), child) == -1))
			{
				child.push_back(second.at(j));
				jpos = j + 1;
				added = jadded = true;
				break;
			}

		//inace nije nasao nikog novog za dodati
		if (!jadded)
			jpos = second.size();

	} while (added);

	child = (*this.*recoverPtr)(child);

	return child;
}

vector<int> GraphRelativeDeviation::crossoverGreedyRelativeDeviationProfitAllVertices(vector<int>& parent1, vector<int>& parent2)
{
	int i;
	vector<int> longChild(numberV, 0);
	double sum1 = 0, sum2 = 0;
	double probability1, probability2;
	double randomValue;
	vector<int> child;

	//ako oboje imaju isti gen tada je rez=1
	//ako samo roditelj1 ima gen tada je rez=-1
	//ako samo roditelj2 ima gen tada je rez=2
	for (i = 0; i < parent1.size(); ++i)
		longChild.at(parent1.at(i)) = -1;

	for (i = 0; i < parent2.size(); ++i)
		longChild.at(parent2.at(i)) = 2;

	sum1 = calculateMaximalRelativeDeviation(parent1);
	sum2 = calculateMaximalRelativeDeviation(parent2);

	//!!!!! zamijenili djeljenike. ako je jako veliko odstupanje, vecu vjerojatnost ima onaj drugi
	probability1 = sum2 / (sum1 + sum2);
	probability2 = sum1 / (sum1 + sum2);

	for (i = 0; i < numberV; ++i)
	{
		if (longChild.at(i) == 1)
			child.push_back(i);
		else
		{
			randomValue = (double)rand() / RAND_MAX;

			if (probability1 <= probability2)
			{
				if (randomValue < probability1) //nasljeduje se gen od roditelja1
				{
					if (longChild.at(i) == -1)
						child.push_back(i);
				}
				else //nasljeduje se gen od roditelja2
				{
					if (longChild.at(i) == 2)
						child.push_back(i);
				}
			}
			else
			{
				if (randomValue < probability2) //nasljeduje se gen od roditelja1
				{
					if (longChild.at(i) == 2)
						child.push_back(i);
				}
				else //nasljeduje se gen od roditelja2
				{
					if (longChild.at(i) == -1)
						child.push_back(i);
				}

			}

		}
	}

	child = (*this.*recoverPtr)(child);

	return child;
}

vector<int> GraphRelativeDeviation::mutationMix(vector<int> someSolution)
{
	int mutationNumb = (int)rand() % 5;

	if (mutationNumb == 0)
		someSolution = mutationDifferenceContribution(someSolution);
	else if (mutationNumb == 1)
		someSolution = mutationLocalSearch(someSolution);
	else if (mutationNumb == 2)
		someSolution = mutationLowProfit(someSolution);
	else if (mutationNumb == 3)
		someSolution = mutationHighProfit(someSolution);
	else if (mutationNumb == 4)
		someSolution = mutationSwap(someSolution);

	return someSolution;
}

vector<int> GraphRelativeDeviation::mutationLowProfit(vector<int> someSolution)
{
	int i;
	int numberOfScenario = (int)rand() % numberS;
	int positionOfSelectedGene = (int)rand() % (someSolution.size());
	int profit = allScenarios[numberOfScenario][someSolution.at(positionOfSelectedGene)];

	for (i = 0; i < numberV; ++i)
		if (profit > allScenarios[numberOfScenario][i])
		{
			someSolution.at(positionOfSelectedGene) = i;
			break;
		}

	//opravimo nakon mutacije
	someSolution = (*this.*recoverPtr)(someSolution);

	return someSolution;
}

vector<int> GraphRelativeDeviation::mutationHighProfit(vector<int> someSolution)
{
	int i;
	int numberOfScenario = (int)rand() % numberS;
	int positionOfSelectedGene = (int)rand() % (someSolution.size());
	int profit = allScenarios[numberOfScenario][someSolution.at(positionOfSelectedGene)];

	for (i = 0; i < numberV; ++i)
		if (profit < allScenarios[numberOfScenario][i])
		{
			someSolution.at(positionOfSelectedGene) = i;
			break;
		}

	//opravimo nakon mutacije
	someSolution = (*this.*recoverPtr)(someSolution);

	return someSolution;
}

vector<int> GraphRelativeDeviation::mutationSwap(vector<int> someSolution)
{
	int first, second, positionFirst, positionSecond;

	first = (int)rand() % (numberV);
	second = (int)rand() % (numberV);
	positionFirst = isElement(first, someSolution);
	positionSecond = isElement(second, someSolution);

	while (((positionFirst == -1) && (positionSecond == -1)) || (positionFirst >= 0 && positionSecond >= 0)) //dok su oba 0 ili oba 1, to ukljucuje i da su ista
	{
		second = (int)rand() % (numberV);
		positionSecond = isElement(second, someSolution);
	}

	if (positionFirst == -1) //prvog nema
	{
		(someSolution).erase(someSolution.begin() + positionSecond);
		someSolution.push_back(first);
	}
	else //drugog nema
	{
		someSolution.erase(someSolution.begin() + positionFirst);
		someSolution.push_back(second);
	}

	//opravimo nakon mutacije
	someSolution = (*this.*recoverPtr)(someSolution);
	return someSolution;
}

vector<int> GraphRelativeDeviation::mutationDifferenceContribution(vector<int> someSolution)
{
	//gledamo rjesenje na ostalom skupu vrhova

	int i, j, position = 0;

	vector<int> otherVertices(numberV - someSolution.size(), 0), otherVerticesContribution(numberV - someSolution.size(), 0),
		otherVerticesNumberOfNeighbours(numberV - someSolution.size(), 0);

	for (i = 0; i < numberV; ++i)
	{
		bool element = false;

		if (isElement(i, someSolution) != -1)
			element = true;

		if (element == false)
		{
			otherVertices[position] = i;
			otherVerticesNumberOfNeighbours[position] = numberOfNeighbours[i];

			//smanjujemo br susjeda ako je netko susjed s elementima prvog skupa
			for (j = 0; j < someSolution.size(); ++j)
				if (adjacencyMatrix[i][someSolution.at(j)])
					--otherVerticesNumberOfNeighbours[position];

			for (j = 0; j < numberS; ++j)
				otherVerticesContribution[position] += allScenarios[j][i];

			if (otherVerticesNumberOfNeighbours[position] != 0)
				otherVerticesContribution[position] = int((double)otherVerticesContribution[position] / otherVerticesNumberOfNeighbours[position] + 0.5);

			position++;
		}
	}

	someSolution = greedyAlgorithmStandardProblem(otherVertices, otherVerticesContribution).first;
	someSolution = (*this.*recoverPtr)(someSolution);
	return someSolution;
}

vector<int> GraphRelativeDeviation::mutationLocalSearch(vector<int> someSolution)
{
	int i, j, k, worstVertex, worstVertexIndex, worstVertexProfit;
	bool conected;

	//ovog pokusamo lokalno popraviti po svim scenarijima

	vector<int> bestSolution = someSolution;
	double bestSolutionDeviation = calculateMaximalRelativeDeviation(bestSolution);

	vector<int> currentSolution, bestChangedSolution;
	double bestChangedSolutionDeviation = DBL_MAX;

	for (i = 0; i < numberS; ++i)
	{
		//trazmi najlosijeg u skupu
		worstVertexIndex = 0;
		worstVertex = bestSolution[0];
		worstVertexProfit = allScenarios[i][worstVertex];

		for (j = 1; j < bestSolution.size(); ++j)
			if (allScenarios[i][bestSolution.at(j)] < worstVertexProfit)
			{
				worstVertexIndex = j;
				worstVertex = bestSolution.at(j);
				worstVertexProfit = allScenarios[i][worstVertex];
			}

		//gledamo jel neki njegov susjed boljeg profita
		for (j = 0; j < numberV; ++j)
			if (adjacencyMatrix[j][worstVertex] && allScenarios[i][j] > worstVertexProfit)
			{
				//ako je onda pogledamo je li povezan s ostalima iz skupa

				conected = false;
				for (k = 0; k < bestSolution.size(); ++k)
					if (adjacencyMatrix[j][bestSolution.at(k)] && (k != worstVertexIndex))
					{
						conected = true;
						break;
					}

				if (!conected) //ako nije, dodaj ga umjesto najlosijeg
				{
					worstVertex = j;
					worstVertexProfit = allScenarios[i][worstVertex];
					//mogli bi sada brejkat, ali damo da jos popravljaju najlosijeg
				}
			}

		currentSolution = bestSolution;
		currentSolution[worstVertexIndex] = worstVertex;

		double pom = calculateMaximalRelativeDeviation(currentSolution);
		if (pom < bestChangedSolutionDeviation)
		{
			bestChangedSolution = currentSolution;
			bestChangedSolutionDeviation = pom;
		}
	}

	if (bestChangedSolutionDeviation < bestSolutionDeviation)
		bestSolution = (*this.*recoverPtr)(bestChangedSolution);
	//ovaj recover jer mozda moze biti jos tko dodan, ali se iz prakse cini da ne moze

	//mozda nesto popravili, mozda ne
	someSolution = bestSolution;
	someSolution = (*this.*recoverPtr)(someSolution);
	return someSolution;
}

pair< vector<int>, double> GraphRelativeDeviation::geneticAlgorithm(int populationType, int crossoverType, int mutationType, int recoverType, int numberOfSteps, int tournamentSize, int badTournamentSize, double mutationParameter)
{
	int  i, temp, randomValue;
	vector<vector<int> > initialPopulation;

	if (populationType == 1)
		populationPtr = &GraphRelativeDeviation::createPopulationExactSolutions;
	else if (populationType == 2)
		populationPtr = &GraphRelativeDeviation::createPopulationGreedyContribution;
	else if (populationType == 3)
		populationPtr = &GraphRelativeDeviation::createPopulationGreedyProfit;
	else if (populationType == 4)
		populationPtr = &GraphRelativeDeviation::createPopulationMix;

	initialPopulation = (*this.*populationPtr)();

	vector<int> newChild;
	pair<int, int> parents;

	pair<vector<int>, double> bestSolution;
	bestSolution.first = initialPopulation.at(0);
	bestSolution.second = calculateMaximalRelativeDeviation(bestSolution.first);

	for (i = 1; i < initialPopulation.size(); ++i)
	{
		double pom = calculateMaximalRelativeDeviation(initialPopulation.at(i));

		if (bestSolution.second > pom)
		{
			bestSolution.first = initialPopulation.at(i);
			bestSolution.second = pom;
		}
	}

	//cout << "naj rel devijacija start " << bestSolution.second << endl;

	if (crossoverType == 1)
		crossoverPtr = &GraphRelativeDeviation::crossoverGreedyContributionAltrenateCombine;
	else if (crossoverType == 2)
		crossoverPtr = &GraphRelativeDeviation::crossoverGreedyContributionAltrenateCombineLocal;
	else if (crossoverType == 3)
		crossoverPtr = &GraphRelativeDeviation::crossoverGreedyContributionAllVertices;
	else if (crossoverType == 4)
		crossoverPtr = &GraphRelativeDeviation::crossoverGreedyRelativeDeviationProfitAllVertices;

	if (mutationType == 1)
		mutationPtr = &GraphRelativeDeviation::mutationDifferenceContribution;
	else if (mutationType == 2)
		mutationPtr = &GraphRelativeDeviation::mutationLocalSearch;
	else if (mutationType == 3)
		mutationPtr = &GraphRelativeDeviation::mutationLowProfit;
	else if (mutationType == 4)
		mutationPtr = &GraphRelativeDeviation::mutationHighProfit;
	else if (mutationType == 5)
		mutationPtr = &GraphRelativeDeviation::mutationSwap;
	else if (mutationType == 6)
		mutationPtr = &GraphRelativeDeviation::mutationMix;

	if (recoverType == 1)
		recoverPtr = &GraphRelativeDeviation::feasibilityRecoverGreedyContribution;
	else if (recoverType == 2)
		recoverPtr = &GraphRelativeDeviation::feasibilityRecoverGreedyContributionAndRandom;
	else if (recoverType == 3)
		recoverPtr = &GraphRelativeDeviation::feasibilityRecoverGreedyProfit;
	else if (recoverType == 4)
		recoverPtr = &GraphRelativeDeviation::feasibilityRecoverGreedyProfitAndRandom;

	for (i = 0; i < numberOfSteps; ++i)
	{
		parents = goodTournament(initialPopulation, tournamentSize);

		newChild = (*this.*crossoverPtr)(initialPopulation.at(parents.first), initialPopulation.at(parents.second));

		if (newChild.size() != 0) 		{
			double pom = calculateMaximalRelativeDeviation(newChild);

			temp = similarity(newChild, initialPopulation, 0.01);
			if (temp != -1)
			{
				initialPopulation.at(temp) = newChild;

				if (pom < bestSolution.second)
				{
					bestSolution.first = newChild;
					bestSolution.second = pom;
					
				}
			}
			else
			{
				temp = badTournament(initialPopulation, badTournamentSize);

				if (pom < calculateMaximalRelativeDeviation(initialPopulation.at(temp)))
				{
					initialPopulation.at(temp) = newChild;

					if (pom < bestSolution.second)
					{
						bestSolution.first = newChild;
						bestSolution.second = pom;
					}
				}
			}
		}

		randomValue = ((int)rand() % 100) + 1; //+1 jer daje od 0
		if (randomValue <= mutationParameter)
		{
			int numberOfSolution = ((int)rand() % initialPopulation.size());
			initialPopulation[numberOfSolution] = (*this.*mutationPtr)(initialPopulation[numberOfSolution]);
			++i; //mutacija se racuna kao posebni korak. 


			double pom = calculateMaximalRelativeDeviation(initialPopulation[numberOfSolution]);

			if (pom < bestSolution.second)
			{
				bestSolution.first = initialPopulation[numberOfSolution];
				bestSolution.second = pom;
			}
		}
	}

	bestSolution.second = calculateMaximalRelativeDeviationReal(bestSolution.first);

	return bestSolution;
}

pair<vector<int>, double> GraphRelativeDeviation::greedyAlgorithmProfit()
{
	int i;
	pair<vector<int>, double> solution;

	for (i = 0; i < numberV; ++i)
	{
		if (!isConected(allVerticesSortedByContribution.at(i), solution.first))
			(solution.first).push_back(allVerticesSortedByContribution.at(i));
	}

	solution.second = calculateMaximalRelativeDeviation(solution.first);

	return solution;
}

pair<vector<int>, double> GraphRelativeDeviation::greedyAlgorithmContribution()
{
	int i;
	pair<vector<int>, double> solution;

	for (i = 0; i < numberV; ++i)
	{
		if (!isConected(allVerticesSortedByProfit.at(i), solution.first))
			(solution.first).push_back(allVerticesSortedByProfit.at(i));
	}

	solution.second = calculateMaximalRelativeDeviation(solution.first);

	return solution;
}

pair<vector<int>, double> GraphRelativeDeviation::greedyAlgorithmStandardProblem(vector<int> vertices, vector<int> criterium)
{

	int i;
	pair<vector<int>, int> solution;

	quickSort(vertices, criterium, 0, vertices.size() - 1, vertices.size());

	for (i = 0; i < vertices.size(); ++i)
		if (!isConected(vertices.at(i), solution.first))
			(solution.first).push_back(vertices.at(i));

	solution.second = calculateMaximalRelativeDeviation(solution.first);

	return solution;
}

double runRelativeExample(string basefileName, int populationType, int crossovertype, int mutationType, int recoverType, int numberOfSteps, int goodTournamentSize, int badTournamentSize, int mutationParameter) //mutacijski parametar ide u postotcima
{
	ofstream outFile;
	string  inputFileName, outputFileName;
	int e, i;

	double time = 0;
	clock_t c_start, c_end;

	outputFileName = "G_relative_solution_" + to_string(crossovertype) + "_" + to_string(mutationType) + "_" + basefileName + ".txt";
	outFile.open(outputFileName, std::ios_base::app);

	if (!outFile.is_open())
		throw "Ne mogu otvoriti datoteku";

	for (e = 1; e <= 30; ++e) //imamo 30 primjera
	{
		inputFileName = basefileName + "_" + to_string(e) + ".txt";
		GraphRelativeDeviation G(inputFileName);

		for (i = 1; i <= 10; ++i) //10 puta vrtimo svaku kombinaciju
		{
			c_start = std::clock();
			pair< vector <int>, double> solution = G.geneticAlgorithm(populationType, crossovertype, mutationType, recoverType, numberOfSteps, goodTournamentSize, badTournamentSize, mutationParameter);
			c_end = std::clock();
			time += (double)(c_end - c_start) / CLOCKS_PER_SEC;

			outFile << solution.second << " ";
		}

		outFile << endl;
	}

	outFile.close();
	time = time / 300;
	return time;
}